@@ -39,10 +39,12 @@ public:
 private Q_SLOTS:
     void initTestCase();
     void testLoadPlugin();
+    void testEmptyModel();
     void testModel();
     void testModelSignals();
     void testModelDisplayName();
     void testProviderModel();
+    void testProviderModelWithApplication();
     void testAccountService();
     void testAccountServiceUpdate();
     void testAuthentication();
@@ -108,6 +110,56 @@ void PluginTest::testLoadPlugin()
     delete object;
 }
 
+void PluginTest::testEmptyModel()
+{
+    clearDb();
+
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import Ubuntu.OnlineAccounts 0.1\n"
+                      "AccountServiceModel {}",
+                      QUrl());
+    QObject *object = component.create();
+    QVERIFY(object != 0);
+    QAbstractListModel *model = qobject_cast<QAbstractListModel*>(object);
+    QVERIFY(model != 0);
+
+    QCOMPARE(model->rowCount(), 0);
+
+    /* We'll now add some accounts but set the service type filter so that they
+     * should not appear in the model. */
+    model->setProperty("serviceType", QString("e-mail"));
+    QCOMPARE(model->property("serviceType").toString(), QString("e-mail"));
+
+    /* Create some disabled accounts */
+    Manager *manager = new Manager(this);
+    Service coolMail = manager->service("coolmail");
+    Service badMail = manager->service("badmail");
+    Account *account1 = manager->createAccount("cool");
+    QVERIFY(account1 != 0);
+
+    account1->setEnabled(false);
+    account1->setDisplayName("CoolAccount");
+    account1->selectService(coolMail);
+    account1->setEnabled(true);
+    account1->syncAndBlock();
+
+    Account *account2 = manager->createAccount("bad");
+    QVERIFY(account2 != 0);
+
+    account2->setEnabled(true);
+    account2->setDisplayName("BadAccount");
+    account2->selectService(badMail);
+    account2->setEnabled(false);
+    account2->syncAndBlock();
+
+    QTest::qWait(10);
+    QCOMPARE(model->rowCount(), 0);
+
+    delete manager;
+    delete object;
+}
+
 void PluginTest::testModel()
 {
     clearDb();
@@ -524,6 +576,7 @@ void PluginTest::testProviderModel()
 
     QCOMPARE(model->rowCount(), providers.count());
     QCOMPARE(model->property("count").toInt(), providers.count());
+    QCOMPARE(model->property("applicationId").toString(), QString());
 
     for (int i = 0; i < providers.count(); i++) {
         QCOMPARE(get(model, i, "displayName").toString(), providers[i].displayName());
@@ -546,6 +599,44 @@ void PluginTest::testProviderModel()
     delete object;
 }
 
+void PluginTest::testProviderModelWithApplication()
+{
+    /* Create some accounts */
+    Manager *manager = new Manager(this);
+    ProviderList providers = manager->providerList();
+
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import Ubuntu.OnlineAccounts 0.1\n"
+                      "ProviderModel {\n"
+                      "  applicationId: \"mailer\"\n"
+                      "}",
+                      QUrl());
+    QObject *object = component.create();
+    QVERIFY(object != 0);
+    QAbstractListModel *model = qobject_cast<QAbstractListModel*>(object);
+    QVERIFY(model != 0);
+
+    QCOMPARE(model->rowCount(), providers.count());
+    QCOMPARE(model->property("count").toInt(), providers.count());
+    QCOMPARE(model->property("applicationId").toString(), QString("mailer"));
+
+    /* Now set an application which supports only "coolservice" and verify that
+     * only the "cool" provider is there */
+    QSignalSpy countSignal(model, SIGNAL(countChanged()));
+    model->setProperty("applicationId", QString("coolpublisher"));
+    QCOMPARE(model->property("applicationId").toString(), QString("coolpublisher"));
+    QCOMPARE(countSignal.count(), 1);
+    QCOMPARE(model->property("count").toInt(), 1);
+    /* Do it twice, just to improve branch coverage */
+    model->setProperty("applicationId", QString("coolpublisher"));
+
+    QCOMPARE(get(model, 0, "providerId").toString(), QString("cool"));
+
+    delete manager;
+    delete object;
+}
+
 void PluginTest::testAccountService()
 {
     clearDb();
